/******************************************************************************
 * bullet object headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *****************************************************************************/


#include "bullet.h"
#include "game.h"
#include "sprite.h"
#include "math.h"
#include "collide.h"
#include "game.h"
#include "debug.h"



/********************************************************************
 * launch a bullet
 *
 * IN:		world positon of the bullet
 * 			speed it travels at
 *
 * RETURNS:	ptr to the bullet object
 *******************************************************************/
BulletStruct *Bullet_New (s24f8_vect2D *pPos, s24f8_vect2D *pSpeed)
{
u32 objID;
BulletStruct *pBullet;

	// grab a new object
	objID = Object_New(OBJECT_BULLET);
	if (objID == OBJECT_ID_NULL)
	{	// error... no unused objects left
		return(NULL);
	}

	// get the pointer to it
	pBullet = (BulletStruct *)Object_getPtrS(objID);

	pBullet->pos = *pPos;
	pBullet->speed = *pSpeed;

	if (pSpeed->x < 0)
	{
		if (pSpeed->y < 0)
		{
			pBullet->tileNum = BULLET_DIAG_TILENUM;
			pBullet->flip = 0;
			pBullet->offset.x = int_to_s24f8(-1);
			pBullet->offset.y = int_to_s24f8(-1);
		}
		else if (pSpeed->y > 0)
		{
			pBullet->tileNum = BULLET_DIAG_TILENUM;
			pBullet->flip = SPRITE_HFLIP;
			pBullet->offset.x = int_to_s24f8(-7);
			pBullet->offset.y = int_to_s24f8(-1);
		}
		else
		{
			pBullet->tileNum = BULLET_HORIZ_TILENUM;
			pBullet->flip = 0;
			pBullet->offset.x = int_to_s24f8(-1);
			pBullet->offset.y = int_to_s24f8(0);
		}
	}
	else if (pSpeed->x > 0)
	{
		if (pSpeed->y < 0)
		{
			pBullet->tileNum = BULLET_DIAG_TILENUM;
			pBullet->flip = SPRITE_HFLIP;
			pBullet->offset.x = int_to_s24f8(-7);
			pBullet->offset.y = int_to_s24f8(-1);
		}
		else if (pSpeed->y > 0)
		{
			pBullet->tileNum = BULLET_DIAG_TILENUM;
			pBullet->flip = 0;
			pBullet->offset.x = int_to_s24f8(-1);
			pBullet->offset.y = int_to_s24f8(-1);
		}
		else
		{
			pBullet->tileNum = BULLET_HORIZ_TILENUM;
			pBullet->flip = 0;
			pBullet->offset.x = int_to_s24f8(-1);
			pBullet->offset.y = int_to_s24f8(0);
		}
	}
	else
	{
		pBullet->tileNum = BULLET_VERT_TILENUM;
		pBullet->flip = 0;
		pBullet->offset.x = int_to_s24f8(0);
		pBullet->offset.y = int_to_s24f8(-1);
	}

	return(pBullet);
}


/****************************************************************
 * force a bullet to stay within the screen boundries
 *
 * IN:		ptr to a position
 *
 * RETURNS:	non-zero if we hit the edge of the world
 ***************************************************************/
u32 Bullet_OffScreen (s24f8_vect2D *pPos)
{
	if (   (pPos->x < int_to_s24f8(1))
		|| (pPos->x > int_to_s24f8(240-1))
		|| (pPos->y < int_to_s24f8(1))
		|| (pPos->y > int_to_s24f8(159)))
	{
		return(1);
	}

	return(0);
}

/**********************************************************************
 * update a bullet object
 *********************************************************************/
u32 Bullet_Update (ObjectStruct *pObj)
{
BulletStruct *pBullet = (BulletStruct *)pObj;
s16 drawPosX, drawPosY;
u32 terrain;

	// move the bullet
	pBullet->pos.x += s24f8_mul(pBullet->speed.x, frameTimeF);
	pBullet->pos.y += s24f8_mul(pBullet->speed.y, frameTimeF);

	if (   (Bullet_OffScreen(&pBullet->pos))
		|| (Collide_BulletToEnemy(&pBullet->pos)))
	{	// bullet hit something or went off screen... kill it
		return(1);
	}

	terrain = Collide_PointToTerrain(&pBullet->pos);
	if ((terrain != TERRAINTYPE_NONE) && (terrain != TERRAINTYPE_LADDER))
	{	// bullet collided with terrain
		return(1);
	}

	drawPosX = s24f8_to_int(pBullet->pos.x + pBullet->offset.x);
	drawPosY = s24f8_to_int(pBullet->pos.y + pBullet->offset.y);

	Sprite_DrawQuickFlip(drawPosX, drawPosY, pBullet->tileNum, BULLET_PALETTE, pBullet->flip);

	return(0);
}


