/********************************************************************
 * game collision
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *******************************************************************/

#ifndef _COLLIDE_H
#define	_COLLIDE_H

#include "types.h"
#include "math.h"
#include "powerup.h"

// conversions from game tile reference to terrain type
#define	TERRAINTILE_SOLID_START		1
#define	TERRAINTILE_NUM_SOLID	  	16
#define	TERRAINTILE_LADDERBASE		15
#define	TERRAINTILE_LADDER			16
#define	TERRAINTILE_PLAYERLAUNCH	17
#define	TERRAINTILE_ENEMYLAUNCH		18
#define	TERRAINTILE_AMMOPOWERUP		19
#define	TERRAINTILE_FINISH			20

// terrain types
typedef enum enumTerrainTypes
{
	TERRAINTYPE_NONE,

	TERRAINTYPE_SOLID,
	TERRAINTYPE_LADDER,
	TERRAINTYPE_LADDERBASE,

	NUM_TERRAINTYPES,
} enumTerrainTypes;


/*****************************************************
 * collide a point with the background terrain
 *
 * IN:		pPos = position to collide against
 *
 * RETURNS:	terrain type (TERRAINTYPE_NONE if no collision)
 ****************************************************/
extern u32 Collide_PointToTerrain (s24f8_vect2D *pPos);

/*************************************************************************
 * collide a powerup to the player
 *
 * IN:		pPowerup = ptr to a powerup
 *
 * RETURNS:	non-zero if the powerup was picked up
 *************************************************************************/
extern u32 Collide_PowerupToPlayer(PowerupStruct *pPowerup);

/*******************************************************************
 * collide a bullet with the enemy dudes
 *
 * IN:		the bullet's position
 *
 * RETURNS:	non-zero if the bullet collided with an enemy
 ******************************************************************/
extern u32 Collide_BulletToEnemy (s24f8_vect2D *pPos);

/*******************************************************************
 * collide the player with the enemy dudes
 *
 * RETURNS:	non-zero if the player collided with an enemy
 ******************************************************************/
extern u32 Collide_PlayerToEnemy (void);


#endif
