/********************************************************************
 * dude game object
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *******************************************************************/

#ifndef _DUDE_H
#define	_DUDE_H

#include "object.h"
#include "math.h"


#define	DUDE_WALK_ANIM_START	0	// start tile of walk anim
#define	DUDE_CLIMB_ANIM_START	4	// start of the climb anim
#define	DUDE_JUMP_ANIM_START	8	// start of the jump anim
#define	DUDE_FIRE_AIM_HORIZ		9	// aim horizontal frame
#define	DUDE_FIRE_AIM_VERT		12	// aim vertical frame
#define	DUDE_FIRE_AIM_DIAGDOWN	11	// aim down frame
#define	DUDE_FIRE_AIM_DIAGUP	10	// aim up frame

typedef enum enumAimDirections
{
	AIM_UP,
	AIM_DIAGUPRIGHT,
	AIM_RIGHT,
	AIM_DIAGDOWNRIGHT,
	AIM_DOWN,
	AIM_DIAGDOWNLEFT,
	AIM_LEFT,
	AIM_DIAGUPLEFT,
} enumAimDirections;


// dude types
typedef enum enumDudeTypes
{
	DUDE_PLAYER,
	DUDE_ENEMY,

	NUM_DUDE_TYPES,
} enumDudeTypes;


// behavior states dude can be in
typedef enum enumDudeStates
{
	DUDE_STATE_WALK,
	DUDE_STATE_CLIMB,
	DUDE_STATE_FIRE,
	DUDE_STATE_FALL,
	DUDE_STATE_DYING,

	DUDE_NUM_STATES,
} enumDudeStates;


// dude character game object data struct
typedef struct DudeStruct
{
	OBJECT_HEADER
	/////////////

	s24f8_vect2D	pos;	// dude's current world position
	s24f8_vect2D	lastFramePos;	// dude's world position at the last animation frame change

	s24f8			gravAccel;		// the current accell speed from gravity

	s24f8			deathTimer;		// timer for death scene

	u8				dudeType;		// what kind of dude is it?

	u8				behaviorState;	// what state is dude in

	s8				aim;			// aiming direction (in firing state only)
	u8				palette;		// palette number
	u16				flip;			// is the dude facing left or right?

	u8				animFrame;		// what anim frame te dude is on

} DudeStruct;


/************************************************************
 * create a new Dude character
 *
 * IN:		pPos = position to launch at
 * 			dudeType = type of dude
 *
 * RETURNS:	ptr to the new Dude game object
 ***********************************************************/
extern DudeStruct *Dude_New (s24f8_vect2D *pPos, u32 dudeType);


/****************************************************************
 * update a dude character
 ***************************************************************/
extern u32 Dude_Update (ObjectStruct *pObj);


#endif
