/**************************************************************
 * External RAM memory allocation system
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *************************************************************/

#include "eramHeap.h"



s32 *gpEramHeapTop = NULL;	// ptr to current top of heap
s32 gEramHeapSize = 0;		// size (in 4 byte words) left in the heap


/*************************************************************
 * init the External-RAM heap
 *
 * NOTES:
 * 		This routine should be called only at level init... it
 * 		will destroy anything that was previously allocated in the
 * 		heap
 ************************************************************/
void EramHeap_InitSystem (void)
{
	// set top of heap to beginning of eram
	gpEramHeapTop = (s32 *)ERAM_TOP;

	// set size to size of eram (in 4 byte words)
	gEramHeapSize = (ERAM_END - ERAM_TOP)>>2;
}


/*************************************************************
 * allocate some data space from the External-RAM heap
 *
 * IN:		size of the data space to allocate (in bytes)
 *
 * RETURNS:	ptr to the top of the allocated data space
 *
 * NOTE:	this routine actually allocates space aligned to
 * 			4 bytes boundries
 *
 * 			The data block returned from this call is uninitialized...
 * 			there is no garantee that the values will be 0!
 *************************************************************/
void *EramHeap_Allocate (u32 byteSize)
{
u32 wordSize;
void *pRet;

	wordSize = byteSize>>2;		// 4 bytes per word
	if (byteSize & 0x03)
	{	// we don't fall on an exact 4 byte word boundry... add an extra word to compensate
		wordSize += 1;
	}

	if (wordSize > gEramHeapSize)
	{	// error... not enough space left in the heap!
		// assert(0);
		return(NULL);
	}

	// top of heap is the location of the new data space
	pRet = gpEramHeapTop;

	// adjust the heap to remove this new data space
	gpEramHeapTop += wordSize;
	gEramHeapSize -= wordSize;

	return(pRet);
}


