/**************************************************************
 * External RAM memory allocation system
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *************************************************************/

#ifndef _ERAMHEAP_H
#define	_ERAMHEAP_H


#include "types.h"
#include "gba_mem.h"


extern s32 gEramHeapSize;		// size (in 4 byte words) left in the heap

/*************************************************************
 * init the External-RAM heap
 *
 * NOTES:
 * 		This routine should be called only at level init... it
 * 		will destroy anything that was previously allocated in the
 * 		heap
 ************************************************************/
extern void EramHeap_InitSystem (void);


/*************************************************************
 * allocate some data space from the External-RAM heap
 *
 * IN:		size of the data space to allocate (in bytes)
 *
 * RETURNS:	ptr to the top of the allocated data space
 *
 * NOTE:	this routine actually allocates space aligned to
 * 			4 bytes boundries
 *
 * 			The data block returned from this call is uninitialized...
 * 			there is no garantee that the values will be 0!
 *************************************************************/
extern void *EramHeap_Allocate (u32 byteSize);


#endif
