/***************************************************************
 * game loop and related game headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 **************************************************************/

#ifndef _GAME_H
#define	_GAME_H


#include "types.h"
#include "math.h"
#include "dat.h"
#include "background.h"
#include "dude.h"

// game modes
typedef enum eGameplayState
{
	GAMEPLAY_NONE,			// no game mode assigned
	GAMEPLAY_NORMAL,		// normal gameplay running
	GAMEPLAY_PAUSED,		// gameplay paused
	GAMEPLAY_GAMEOVER,		// display winning information
	GAMEPLAY_SHELL_LOGO,	// display the shell logo
	GAMEPLAY_SHELL_MENU,	// display the shell menus

} eGameplayState;


// number of vblanks the previous frame took
extern u16 frameTime;
extern u16 frameTimeAccum;
extern s24f8 frameTimeF;

extern u16 gameplayState;		// the game state the game is in currently
extern u16 gameplayNewState;	// set this variable to the new game state to go to

extern u32 gGameLevel;
extern BackgroundFrameStruct *gpGameLevelMap;

// global for the player struct
extern DudeStruct *gpPlayerDude;
u32 gPlayerAmmo;
u32 gPlayerLives;


/**************************************************************
 * convert hours, minutes and seconds to a 32 bit int
 *************************************************************/
#define	time_to_int(hours,minutes,seconds)	((hours<<16) + (minutes<<8) + seconds)

/**************************************************************
 * convert a 32 bit int to hours, minutes and seconds (assumes time value is valid!)
 *************************************************************/
#define	time_hours(time)	((time>>16) & 0x0FF)
#define	time_minutes(time)	((time>>8) & 0x0FF)
#define	time_seconds(time)	(time & 0x0FF)


/**************************************************************
 * the master game loop
 *************************************************************/
extern void Game_MainLoop (void);




// game settings

// sprite tile locations in VRAM
#define	DUDE_WALK_TILENUM			0
#define	DUDE_CLIMB_TILENUM			(DUDE_WALK_TILENUM + (DUDE_WALK_NUM_FRAMES*2))
#define	DUDE_JUMP_TILENUM			(DUDE_CLIMB_TILENUM + (DUDE_CLIMB_NUM_FRAMES*2))
#define	DUDE_FIRE_HORIZ_TILENUM		(DUDE_JUMP_TILENUM + 2)
#define	DUDE_FIRE_DIAGUP_TILENUM	(DUDE_FIRE_HORIZ_TILENUM + 2)
#define	DUDE_FIRE_DIAGDOWN_TILENUM	(DUDE_FIRE_DIAGUP_TILENUM + 2)
#define	DUDE_FIRE_VERT_TILENUM		(DUDE_FIRE_DIAGDOWN_TILENUM + 2)
#define	BULLET_HORIZ_TILENUM		(DUDE_FIRE_VERT_TILENUM + 2)
#define	BULLET_DIAG_TILENUM			(BULLET_HORIZ_TILENUM + 1)
#define	BULLET_VERT_TILENUM			(BULLET_DIAG_TILENUM + 1)

#define	NUMS_TILENUM				(BULLET_VERT_TILENUM + 1)
#define	EXCLAMATION_TILENUM			(NUMS_TILENUM + 10)
#define	QUESTIONMARK_TILENUM		(EXCLAMATION_TILENUM + 1)
#define	PERIOD_TILENUM				(QUESTIONMARK_TILENUM + 1)
#define	COMMA_TILENUM				(PERIOD_TILENUM + 1)
#define	LETTERS_TILENUM				(COMMA_TILENUM + 1)
#define	DASH_TILENUM				(LETTERS_TILENUM + 26)
#define	COLON_TILENUM				(DASH_TILENUM + 1)

#define	HEART_TILENUM				(COLON_TILENUM + 1)
#define	SKULL_TILENUM				(HEART_TILENUM + 1)
#define	BOX_TILENUM					(SKULL_TILENUM + 1)
#define	CLOCK_TILENUM				(BOX_TILENUM + 1)
#define	AMMO_TILENUM				(CLOCK_TILENUM + 1)
#define	ARROW_TILENUM				(AMMO_TILENUM + 1)
#define	ARROWV_TILENUM				(ARROW_TILENUM + 1)
#define	A_BUTTON_TILENUM			(ARROWV_TILENUM + 1)

// sprite palette locations
#define	DUDE_PALETTE	0
#define	ENEMY_PALETTE	1
#define	DEREZ_PALETTE	2
#define	BULLET_PALETTE	3
#define	POWERUP_PALETTE	4
#define	FONT_PALETTE	5

// 42 characters in the font file, and 8 extra icons
#define	NUM_FONT_TILES				42
#define	NUM_ICON_TILES				8

// game text that is needed outside the menus....
extern u8 msg_pause[];
extern u8 msg_playerdead[];
extern u8 msg_pressStart[];

#endif
