/*************************************************************************
 * GBA Direct Memory Access headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 ************************************************************************/

#ifndef _GBA_DMA_H
#define	_GBA_DMA_H


// DMA registers
#define pREG_DMA_0_SRC_ADDR		(u32 *)0x40000B0
#define pREG_DMA_0_DEST_ADDR	(u32 *)0x40000B4
#define pREG_DMA_0_CONTROL		(u32 *)0x40000B8
#define REG_DMA_0_SRC_ADDR		*pREG_DMA_0_SRC_ADDR
#define REG_DMA_0_DEST_ADDR		*pREG_DMA_0_DEST_ADDR
#define REG_DMA_0_CONTROL		*pREG_DMA_0_CONTROL

#define pREG_DMA_1_SRC_ADDR		(u32 *)0x40000BC
#define pREG_DMA_1_DEST_ADDR	(u32 *)0x40000C0
#define pREG_DMA_1_CONTROL		(u32 *)0x40000C4
#define REG_DMA_1_SRC_ADDR		*pREG_DMA_1_SRC_ADDR
#define REG_DMA_1_DEST_ADDR		*pREG_DMA_1_DEST_ADDR
#define REG_DMA_1_CONTROL		*pREG_DMA_1_CONTROL

#define pREG_DMA_2_SRC_ADDR		(u32 *)0x40000C8
#define pREG_DMA_2_DEST_ADDR	(u32 *)0x40000CC
#define pREG_DMA_2_CONTROL		(u32 *)0x40000D0
#define REG_DMA_2_SRC_ADDR		*pREG_DMA_2_SRC_ADDR
#define REG_DMA_2_DEST_ADDR		*pREG_DMA_2_DEST_ADDR
#define REG_DMA_2_CONTROL		*pREG_DMA_2_CONTROL

#define pREG_DMA_3_SRC_ADDR		(u32 *)0x40000D4
#define pREG_DMA_3_DEST_ADDR	(u32 *)0x40000D8
#define pREG_DMA_3_CONTROL		(u32 *)0x40000Dc
#define REG_DMA_3_SRC_ADDR		*pREG_DMA_3_SRC_ADDR
#define REG_DMA_3_DEST_ADDR		*pREG_DMA_3_DEST_ADDR
#define REG_DMA_3_CONTROL		*pREG_DMA_3_CONTROL

// DMA control register bits
#define	DMA_WORDCOUNT_MASK			0x0000FFFF	// lower word of control register is the DMA word count

#define	DMA_SRC_SETTING_MASK		0x01800000	// mask for the source settings
#define DMA_SRC_INCREMENT			0x00000000	// increment the source address after each transfer
#define DMA_SRC_DECREMENT			0x00800000	// decrement the source address after each transfer
#define DMA_SRC_FIXED				0x01000000	// address stays fixed after each transfer (useful for fills)

#define	DMA_DEST_SETTING_MASK		0x00600000	// mask for the destination settings
#define DMA_DEST_INCREMENT			0x00000000	// increment destination address after each transfer
#define DMA_DEST_DECREMENT			0x00200000	// decrement destination address after each transfer
#define DMA_DEST_FIXED				0x00400000	// address stays fixed after each transfer (useful for direct sound FIFO)
#define	DMA_DEST_INC_RESET			0x00600000	// works like DMA_DEST_INCREMENT, but resets to start address when DMA finishes (for auto-repeating DMAs)

#define DMA_REPEAT					0x02000000	// if set, DMA will auto-repeat every Hblank or Vblank (depending on timing bits)

#define DMA_16BIT_COPY				0x00000000	// copy 16 bit words each transfer
#define DMA_32BIT_COPY				0x04000000	// copy 32 bit words each transfer

#define	DMA_TIMING_MASK				0x30000000	// mask for the DMA timing bits
#define DMA_TIMING_IMMEDIATE		0x00000000	// start the DMA immediately
#define DMA_TIMING_VBLANK			0x10000000	// start the DMA when the next VBlank occurs
#define DMA_TIMING_HBLANK			0x20000000	// start the DMA when the next HBlank occurs
#define DMA_TIMING_SYNC_TO_DISPLAY	0x30000000	// special mode, useful for mode 3 graphics (only works on DMA channel 3)
#define DMA_START_ON_FIFO_EMPTY		0x30000000	// used for timer driven direct sound dma

#define DMA_INTERUPT_ENABLE			0x40000000	// if set, end of DMA generates an interrupt
#define DMA_ENABLE					0x80000000	// when set, the DMA will begin


#endif

