/******************************************************************************
 * powerup object headers
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *****************************************************************************/

#ifndef _POWERUP_H
#define	_POWERUP_H

#include "types.h"
#include "math.h"
#include "object.h"
#include "dude.h"


typedef enum enumPowerupTypes
{
	POWERUP_AMMO,			// ammo powerup
	POWERUP_FINISH,			// game finish powerup

	NUM_POWERUP_TYPES,
} enumPowerupTypes;

typedef struct PowerupStruct
{
	OBJECT_HEADER
	/////////////

	s24f8_vect2D	pos;		// world position of the bullet

	u16				powerupType;	// type of powerup

	u16				value;		// the value of the powerup

} PowerupStruct;


/********************************************************************
 * launch a powerup
 *
 * IN:		type = type of powerup
 * 			pPos = world position of the powerup
 * 			value = value of the powerup (for example, for a health powerup, it is
 * 					the amount of health the powerup gives if picked up)
 *
 * RETURNS:	ptr to the powerup object
 *******************************************************************/
extern PowerupStruct *Powerup_New (u16 type, s24f8_vect2D *pPos, u16 value);


/**********************************************************************
 * update a powerup object
 *********************************************************************/
extern u32 Powerup_Update (ObjectStruct *pObj);


/*****************************************************************
 * give a powerup to a dude
 *
 * IN:		ptr to the dude
 * 			ptr to the powerup
 ****************************************************************/
extern void Powerup_GiveToDude (DudeStruct *pDude, PowerupStruct *pPowerup);


#endif


