/*********************************************************************
 * code for world setup
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *********************************************************************/


#include "world.h"
#include "collide.h"
#include "game.h"
#include "gba_bkg.h"
#include "background.h"

// the position of the screen in the main game world
s24f8_vect2D gWorldPos;
s24f8_vect2D gWorldSize;


/*****************************************************************
 * init the game world
 *
 * IN:		worldX/Y - initial screen position in the world
 * 			worldSizeX/Y - size of the world
 ****************************************************************/
void World_Init (s24f8 worldX, s24f8 worldY, s24f8 worldSizeX, s24f8 worldSizeY)
{
	gWorldSize.x = worldSizeX;
	gWorldSize.y = worldSizeY;

	gWorldPos.x = worldX;
	gWorldPos.y = worldY;
}



/********************************************************
 * luanch the game world and draw it to VRAM
 *******************************************************/
void Level_LaunchAndDraw (void)
{
u32 x, y;
const u16 *pMap;
s24f8_vect2D pos;
u16 drawTile;

	pMap = gpGameLevelMap->pMap;

	for (y = 0; y < 20; y++)
	{
		for (x = 0; x < 30; x++)
		{
			drawTile = pMap[(y*30) + x];

			switch (drawTile & TILE_IDX_MASK)
			{
			case TERRAINTILE_PLAYERLAUNCH:
				// launch the player
				pos.x = int_to_s24f8((x<<3)+4);
				pos.y = int_to_s24f8(y<<3);
				gpPlayerDude = Dude_New(&pos, DUDE_PLAYER);

				drawTile = 0;	// don't draw this tile
				break;

			case TERRAINTILE_ENEMYLAUNCH:
				// launch an enemy
				pos.x = int_to_s24f8((x<<3)+4);
				pos.y = int_to_s24f8(y<<3);
				Dude_New(&pos, DUDE_ENEMY);

				drawTile = 0;	// don't draw this tile
				break;

			case TERRAINTILE_AMMOPOWERUP:
				// launch a ammo powerup
				pos.x = int_to_s24f8(x<<3);
				pos.y = int_to_s24f8(y<<3);
				Powerup_New(POWERUP_AMMO, &pos, 5);

				drawTile = 0;	// don't draw this tile
				break;

			case TERRAINTILE_FINISH:
				// launch a finish powerup
				pos.x = int_to_s24f8(x<<3);
				pos.y = int_to_s24f8((y<<3)+1);
				Powerup_New(POWERUP_FINISH, &pos, 5);

				drawTile = 0;	// don't draw this tile
				break;

			default:
				break;
			}

			// send the tile to VRAM
			gBkg0.pMap[(y*32) + x] = drawTile;
		}
	}
}



